# -*- coding: utf-8 -*-

# Vendor Library
from django.db.models import Q

# Project Library
from kinder.core.direct.models import Direct
from kinder.core.group.models import Pupil


from . import BaseIndex
from ..algorithm.direct import *
from ..constants import ALL
from ..utils import AgeDeltas


class DirectIndex(BaseIndex):
    """Запросы расчитывающие количество направлений,
    написанные на django queryset'ах"""

    FILTER_193 = 1
    FILTER_291 = 2
    FILTER_32 = 3

    ALGORITHM_MAP = {
        FILTER_193: get_193_filter,
        FILTER_291: get_291_filter,
        FILTER_32: get_34_filter,
    }

    def __init__(self, unit):
        self.unit = unit

    def get_count(self, filter_type, **kwargs):

        qs = Direct.objects.filter(
            self.get_algorithm(filter_type)(
                unit=self.unit, **kwargs))

        age_range = kwargs.get('age_range', ALL)
        if age_range != ALL:
            on_date = kwargs.get('on_date', AgeDeltas.calculate_date)
            down, up = AgeDeltas.get_category_deltas(age_range, on_date)
            qs = qs.filter(
                declaration__children__date_of_birth__gt=down,
                declaration__children__date_of_birth__lte=up)

        return qs.filter(group__unit=self.unit).count()


def ind_29_1(dou):
    u"""Показатель #29.1 Прогнозируемое уменьшение контингента."""

    current_learn_year_begin = DateMixin._get_current_calendar_year()

    children = Pupil.objects.filter(grup__unit=dou).exclude(
        temporary_deduct=True).exclude(grup__status=GroupStatusEnum.ARCHIVE)

    return Direct.objects.filter(
        status__code__in=(DRS.REGISTER, DRS.DOGOVOR),
        group__status__in=(GroupStatusEnum.FACT, GroupStatusEnum.PLAN),
    ).filter(
        Q(declaration__desired_date__lte=current_learn_year_begin) |
        Q(declaration__desired_date__isnull=True)
    ).exclude(
        declaration__children__date_of_birth__gte=current_learn_year_begin
    ).filter(
        declaration__children_id__in=children.values_list(
            'children_id', flat=True)
    ).values_list(
        'declaration__children_id', 'declaration__children__date_of_birth')
